 /*
$Id: echoprofile.inc 1.13 2006/11/10 08:28:07EST Basson, Dominique (Dominique Basson) Dev  $
$ProjectName: d:/elect/source/device_descriptions/hart/lr460/pdm/lr460_hart_pdm.pj $   $Locker:  $
*/

#ifndef ECHOPROFILE_INC
#define ECHOPROFILE_INC
#endif

/*                                                       For Internal Use Only.
   Description ----------------------------------------------------------------
       Library File For echo profile.

       The only method to use is "method_menu_echo_profile" the remaining items
       are support elements

       The "echoprofileopt.dd" file must be created.
       Use the template with the name "echoprofileopt.tpl.dd".

       Saving and showing of profiles can be supported by also adding "saveechoprofile.inc".

         Copyright (C) Siemens-Milltronics Process Instruments Inc.
         All Rights Reserved.

   Warnings -------------------------------------------------------------------

   Special Revision Notes -----------------------------------------------------
      To add this feature to a DD:

      1) Add "method_menu_echo_profile" under the "View" menu
      2) Make sure the defines the options file reference the correct variables
      3) Make sure the dictionary entries from the options file are defined
      4) Include "echoprofile.inc" (this file) in the main ddl file
      5) Add "echoprofile.inc" (this file) to the devices file
      6) Add "echoprofileopt.dd" to the devices file

   Defines --------------------------------------------------------------------*/
#define ECHO_PROFILE_NOT_TAKEN 0
#define ECHO_PROFILE_TAKEN     1

/* Include Files --------------------------------------------------------------*/
#include "echoprofileopt.dd"

/* Collections ----------------------------------------------------------------*/

/* Relations (Unit, Array, Refresh) -------------------------------------------*/
UNIT echo_profile_units
{
   ECHO_PROFILE_X_UNITS
   :
   var_local_window_sill_start,
   var_local_window_sill_end,
   var_local_echo_marker_distance
}

/* Variable Definitions -------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_echo_profile_taken
{
   CLASS LOCAL;
   TYPE  UNSIGNED_INTEGER(1)
      {
      DEFAULT_VALUE ECHO_PROFILE_NOT_TAKEN;
      }
}

/*-----------------------------------------------------------------------------
   Description: Variables to display echo marker
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_echo_marker_bottom
{
   CLASS LOCAL;
   TYPE FLOAT
   {
      DEFAULT_VALUE 0;
   }
   CONSTANT_UNIT ECHO_PROFILE_Y_UNITS_LABEL;
}

VARIABLE var_local_echo_marker_top
{
   CLASS LOCAL;
   TYPE FLOAT
   {
      DEFAULT_VALUE 0;
   }
   CONSTANT_UNIT ECHO_PROFILE_Y_UNITS_LABEL;
}

VARIABLE var_local_echo_marker_distance
{
   CLASS LOCAL;
   TYPE FLOAT
   {
      DEFAULT_VALUE 0.0;
   }
}

/*-----------------------------------------------------------------------------
   Description: Variables to display window sill
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_window_sill_offset
{
   CLASS LOCAL;
   TYPE UNSIGNED_INTEGER
   {
      DEFAULT_VALUE 0;
   }
}

VARIABLE var_local_window_sill
{
   CLASS LOCAL;
   TYPE UNSIGNED_INTEGER
   {
      DEFAULT_VALUE 0;
   }
}

VARIABLE var_local_window_sill_start
{
   CLASS LOCAL;
   TYPE FLOAT
   {
      DEFAULT_VALUE 0.0;
   }
}

VARIABLE var_local_window_sill_end
{
   CLASS LOCAL;
   TYPE FLOAT
   {
      DEFAULT_VALUE 0.0;
   }
}

/* Command Definitions --------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/

/* Menu Definitions -----------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU dialog_echo_profile
{
   LABEL  ECHO_PROFILE_DIALOG_LABEL;
   ACCESS OFFLINE;
   STYLE_DIALOG
   ITEMS
   {
      page_echo_profile
   }
}

MENU page_echo_profile
{
   LABEL ECHO_PROFILE_DIALOG_LABEL;
   STYLE_PAGE
   ITEMS
   {
      graph_echo_profile,
      COLUMNBREAK,
      ECHO_PROFILE_PAR_READING         ( READ_ONLY ),
      ECHO_PROFILE_PAR_DISTANCE        ( READ_ONLY ),
      ECHO_PROFILE_PAR_CONFIDENCE      ( READ_ONLY ),
      ECHO_PROFILE_PAR_ECHO_STRENGTH   ( READ_ONLY ),
      ECHO_PROFILE_PAR_NEAR_BLANKING   ( READ_ONLY ),
      ECHO_PROFILE_PAR_ALGORITHM       ( READ_ONLY ),
#ifdef SAVEECHOPROFILE_INC
      group_list_echo_profile,
#endif
      group_measure_echo_profile
   }
}

MENU group_measure_echo_profile
{
   LABEL ECHO_PROFILE_MEASURE_GROUP_LABEL;
   STYLE_GROUP
   ITEMS
   {
      ECHO_PROFILE_DEVICE_STATUS,
      method_measure_echo_profile
   }
}

/* Visual component Definitions -----------------------------------------------*/

/*-----------------------------------------------------------------------------
   Description: Displays graph area, content is referenced from here
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
#ifdef EDDL_SUPPORT
GRAPH graph_echo_profile
{
   X_AXIS x_axis_echo_profile;
   MEMBERS
      {
      WAVEFORM_ECHO_PROFILE_ECHO,   waveform_echo_profile;
      WAVEFORM_ECHO_PROFILE_TVT,    waveform_tvt;
      WAVEFORM_ECHO_PROFILE_MARKER, waveform_profile_marker;
      WAVEFORM_ECHO_PROFILE_WINDOW, waveform_window_marker;
      }
   WIDTH LARGE;
   HEIGHT X_LARGE;
   VALIDITY (var_local_echo_profile_taken == ECHO_PROFILE_TAKEN);
}
#endif

#ifdef EDDL_SUPPORT
WAVEFORM waveform_echo_profile
{
   LINE_COLOR ECHO_PROFILE_WAVEFORM_ECHO_PROFILE_COLOR;
   Y_AXIS     y_axis_echo_profile;
   TYPE       YT
      {
      X_INITIAL        ECHO_PROFILE_X_MIN;
      X_INCREMENT      ECHO_PROFILE_ECHO_POINT_INCREMENT;
      NUMBER_OF_POINTS ECHO_PROFILE_NUMBER_ECHO_POINTS;
      Y_VALUES
         {
         ECHO_PROFILE_ECHO_POINTS
         }
      }
   INIT_ACTIONS
      {
      method_measure_echo_profile
      }
}

WAVEFORM waveform_tvt
{
   LINE_COLOR ECHO_PROFILE_WAVEFORM_TVT_COLOR;
   Y_AXIS     y_axis_echo_profile;
   TYPE       YT
      {
      X_INITIAL        ECHO_PROFILE_X_MIN;
      X_INCREMENT      ECHO_PROFILE_TVT_POINT_INCREMENT;
      NUMBER_OF_POINTS ECHO_PROFILE_NUMBER_TVT_POINTS;
      Y_VALUES
         {
          ECHO_PROFILE_TVT_POINTS
         }
      }
   INIT_ACTIONS
      {
      method_measure_echo_profile
      }
}

/* draw the selected echo marker */
WAVEFORM waveform_profile_marker
{
   EMPHASIS   TRUE;
   LINE_COLOR ECHO_PROFILE_WAVEFORM_PROFILE_MARKER_COLOR;
   Y_AXIS     y_axis_echo_profile;
   TYPE       XY
      {
      X_VALUES
         {
         var_local_echo_marker_distance, var_local_echo_marker_distance
         }
      Y_VALUES
         {
         var_local_echo_marker_top,      var_local_echo_marker_bottom
         }
      NUMBER_OF_POINTS 2;
      }
}

/* draw the window sill marker */
WAVEFORM waveform_window_marker
{
   LINE_COLOR ECHO_PROFILE_WAVEFORM_WINDOW_MARKER_COLOR;
   Y_AXIS     y_axis_echo_profile;
   TYPE       XY
      {
      X_VALUES
         {
         var_local_window_sill_start,  var_local_window_sill_start, var_local_window_sill_end, var_local_window_sill_end
         }
      Y_VALUES
         {
         var_local_window_sill_offset, var_local_window_sill,       var_local_window_sill,     var_local_window_sill_offset
         }
      NUMBER_OF_POINTS 4;
      }
}

AXIS y_axis_echo_profile
{
    LABEL         ECHO_PROFILE_AXIS_PROFILE_LABEL;
    MIN_VALUE     ECHO_PROFILE_Y_MIN;
    MAX_VALUE     ECHO_PROFILE_Y_MAX;
    CONSTANT_UNIT ECHO_PROFILE_Y_UNITS_LABEL;
}

AXIS x_axis_echo_profile
{
    LABEL         ECHO_PROFILE_AXIS_DISTANCE_LABEL;
    MIN_VALUE     ECHO_PROFILE_X_MIN;
    MAX_VALUE     ECHO_PROFILE_X_MAX;
    CONSTANT_UNIT ECHO_PROFILE_X_UNITS(ECHO_PROFILE_X_UNITS);
}
#endif

/* Method Definitions ---------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description    Echo Profile Window
   Created by:    Valerie McQueen
   Revised by:    Gergana Karadjova
 ------------------------------------------------------------------------------*/
METHOD method_menu_echo_profile
{
   LABEL  ECHO_PROFILE_DIALOG_LABEL;
   ACCESS OFFLINE;
   CLASS  INPUT;
   DEFINITION
   {
      long result;
      MenuDisplay(dialog_echo_profile, ECHO_PROFILE_CLOSE_BUTTON, result);
   }
}

/*-----------------------------------------------------------------------------
   Description    Echo Profile Window
   Created by:    Valerie McQueen
   Revised by:    Gergana Karadjova
 ------------------------------------------------------------------------------*/
METHOD method_measure_echo_profile
{
  LABEL ECHO_PROFILE_MEASURE_METHOD_LABEL;
  DEFINITION
  {
      ECHO_PROFILE_READ_ECHO_PROFILE_DATA_METHOD();

      // Compute echo marker values
      var_local_echo_marker_bottom            = ECHO_PROFILE_Y_MIN;
      var_local_echo_marker_top               = var_local_echo_marker_bottom + ECHO_PROFILE_PAR_CONFIDENCE;
      var_local_echo_marker_distance          = ECHO_PROFILE_PAR_DISTANCE;

      // Compute window sill values
      var_local_window_sill                   = ECHO_PROFILE_PAR_WINDOW_SILL;
      var_local_window_sill_offset            = var_local_window_sill + 2;

      var_local_window_sill_start             = ECHO_PROFILE_PAR_DISTANCE - (ECHO_PROFILE_PAR_ECHO_LOCK_WINDOW / 2.0);
      var_local_window_sill_end               = ECHO_PROFILE_PAR_DISTANCE + (ECHO_PROFILE_PAR_ECHO_LOCK_WINDOW / 2.0);

      var_local_echo_profile_taken            = ECHO_PROFILE_TAKEN;
#ifdef SAVEECHOPROFILE_INC
      method_reset_save_profile();
#endif
  }
}

/*-----------------------------------------------------------------------------
                                                         For Internal Use Only.
      Copyright (C) Siemens-Milltronics Process Instruments Inc.
      All Rights Reserved.

End of file -------------------------------------------------------------------*/
