/*
$Id: tvtshaper120.inc 1.10 2006/10/16 13:39:08EDT Pathak, Pranjal (Pranjal Pathak) Dev  $
$ProjectName: d:/elect/source/device_descriptions/hart/lr460/pdm/lr460_hart_pdm.pj $   $Locker:  $
*/
#ifndef TVTSHAPER120_INC
#define TVTSHAPER120_INC
#endif
/*                                                       For Internal Use Only.
   Description ----------------------------------------------------------------
       Library File For 120 point TVT Shaper.

         This is an EDD feature to support SMPI Standard 1200 point manual TVT shapers

         The only method to use is "method_menu_tvt_shaper" the remaining items 
         are support elements

         The basic structure:
                             method_menu_tvt_shaper 
                                        |
                             menu_dialog_tvt_shaper
                                        |
                   -------------------------------------------------------------------------------------------
                   |               |                               |                       |                |
    menu_page_tvt_shaper_graph  menu_page_tvt_shaper_list1 ...tvt_shaper_list2  ...tvt_shaper_list3 ...tvt_shaper_list4
                        |
             ---------------------------------------------------
             |                        |                        |
      graph_tvt_shaper   menu_group_tvt_presettings     menu_group_tvt_shaper,


         Copyright (C) Siemens-Milltronics Process Instruments Inc.
         All Rights Reserved.

   Warnings -------------------------------------------------------------------

   Special Revision Notes -----------------------------------------------------
      To add this feature to a DD:

      1) Add "method_menu_tvt_shaper" under the "Device" menu    
      2) Make sure the defines the options file reference the correct variables
      3) Make sure the dictionary entries from the options file are defined
      4) Include "tvtshaper40.inc" (this file) in the main ddl file
      5) Add "tvtshaper40.inc" (this file) to the devices file
      6) Add "tvtshaper40opt.dd" to the devices file

   Defines --------------------------------------------------------------------*/
#define NUMBER_TVT_SHAPER_POINTS                     120

// Used to define array sizes and x axis bounds
#define NUMBER_TVT_SHAPER_EXTRA_POINTS     8
// The following value must be a number, PDM has problems if it uses an operator
// Set to NUMBER_TVT_SHAPER_POINTS + NUMBER_TVT_SHAPER_EXTRA_POINTS + 1
#define NUMBER_TVT_SHAPER_WORKING_POINTS  129

// Tvt shaper modes
#define TVT_SHAPER_MODE_OFF 0
#define TVT_SHAPER_MODE_ON  1

/* Include Files --------------------------------------------------------------*/
#include "tvtShaper120Opt.dd"

/* Collections ----------------------------------------------------------------*/

/* Relations (Unit, Array, Refresh) -------------------------------------------*/
REFRESH refresh_tvt_shaper_array
{
   var_local_tvt_shaper_index_display
   :
   var_local_tvt_shaper_value_display
}

/* Variable Definitions -------------------------------------------------------*/
var_backup_tvt_shaper_mode     LIKE VARIABLE TVT_SHAPER_MODE  {}

ARRAY array_backup_tvt_shaper_points
{
   TYPE                TVT_SHAPER_Y_MIN;
   NUMBER_OF_ELEMENTS  NUMBER_TVT_SHAPER_POINTS;
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_profile_taken
{
   CLASS       LOCAL;
   TYPE        UNSIGNED_INTEGER(1)
   {
      DEFAULT_VALUE FALSE;
   }
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_index
{
   CLASS       LOCAL;
   TYPE        UNSIGNED_INTEGER(1)
   {
      DEFAULT_VALUE 0;
      MIN_VALUE     0;
      MAX_VALUE     NUMBER_TVT_SHAPER_POINTS - 1;
   }
   HANDLING    READ & WRITE;
}

/*-----------------------------------------------------------------------------
   Description: The index of the value currently being edited    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_index_display
{
   LABEL       VAR_LOCAL_TVT_SHAPER_INDEX_LABEL;
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      DEFAULT_VALUE 1;
      MIN_VALUE     1;
      MAX_VALUE     NUMBER_TVT_SHAPER_POINTS;
   }
   PRE_EDIT_ACTIONS
	{
       method_set_tvt_shaper_index_display
	}
	POST_EDIT_ACTIONS
	{
       method_get_tvt_shaper_index_display
	}
   HANDLING    READ & WRITE;
}

/*-----------------------------------------------------------------------------
   Description: The value currently being edited    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_value_display
{
   LABEL       VAR_LOCAL_TVT_SHAPER_VALUE_LABEL;
   CLASS       LOCAL;
   TYPE        INTEGER(1)
   {
      DEFAULT_VALUE 0;
      MIN_VALUE     TVT_SHAPER_MINIMUM_VALUE;
      MAX_VALUE     TVT_SHAPER_MAXIMUM_VALUE;
   }
   PRE_EDIT_ACTIONS
	{
       method_set_tvt_shaper_offset_display
	}
	POST_EDIT_ACTIONS
	{
       method_get_tvt_shaper_offset_display
	}
   HANDLING    READ & WRITE;
}

/*-----------------------------------------------------------------------------
   Description: 
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_start
{
   TYPE FLOAT;
   CLASS LOCAL;
}

/*-----------------------------------------------------------------------------
   Description: 
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
VARIABLE var_local_tvt_shaper_x_axis_echo_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

VARIABLE var_local_tvt_shaper_tvt_increment
{
   TYPE FLOAT;
   CLASS LOCAL;
}

/*-----------------------------------------------------------------------------
   Description: 
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
ARRAY array_local_tvt_profile
{
   TYPE                TVT_SHAPER_Y_MIN;
   NUMBER_OF_ELEMENTS  NUMBER_TVT_SHAPER_WORKING_POINTS;
}

/* Command Definitions --------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/

/* Menu Definitions -----------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: Displays the tvt shaper dialog   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_dialog_tvt_shaper
{
   LABEL      MENU_DIALOG_TVT_SHAPER_LABEL;
   STYLE_DIALOG
   ACCESS     OFFLINE;
   ITEMS
   {
      menu_page_tvt_shaper_graph,
      menu_page_tvt_shaper_list1,
		menu_page_tvt_shaper_list2,
		menu_page_tvt_shaper_list3,
		menu_page_tvt_shaper_list4
   }
}

/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a graphical editing interface   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_page_tvt_shaper_graph
{
   LABEL      MENU_PAGE_TVT_SHAPER_GRAPH_LABEL;
   STYLE_PAGE
   ACCESS     OFFLINE;
   ITEMS
   {
#ifdef EDDL_SUPPORT
      graph_tvt_shaper,
#endif
      menu_group_tvt_presettings,
      menu_group_tvt_shaper
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for refreshing the profiles   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_group_tvt_presettings
{
   LABEL      MENU_GROUP_TVT_PRESETTINGS_LABEL;
   STYLE_GROUP
   ITEMS
   {
      method_tvt_shaper_update_from_device
   }
}

/*-----------------------------------------------------------------------------
   Description: displays a group on the graph page for editing the TVT   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_group_tvt_shaper
{
   LABEL      MENU_GROUP_TVT_SHAPER_LABEL;
   STYLE_GROUP
   ACCESS     OFFLINE;
   ITEMS
   {
      var_local_tvt_shaper_index_display,    
      method_tvt_index_plus,
      method_tvt_index_minus,
      COLUMNBREAK,
      var_local_tvt_shaper_value_display,
      method_tvt_value_increase,
      method_tvt_value_decrease
   }
}

/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a list of tvt points to be edited   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_page_tvt_shaper_list1
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "1 - 30";
   STYLE_PAGE
   ACCESS      OFFLINE;
   ITEMS
   {
      TVT_SHAPER_ARRAY[ 0] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 1] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 2] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 3] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 4] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 5] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 6] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 7] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 8] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 9] (READ_ONLY),
      TVT_SHAPER_ARRAY[10] (READ_ONLY),
      TVT_SHAPER_ARRAY[11] (READ_ONLY),
      TVT_SHAPER_ARRAY[12] (READ_ONLY),
      TVT_SHAPER_ARRAY[13] (READ_ONLY),
      TVT_SHAPER_ARRAY[14] (READ_ONLY),
		COLUMNBREAK,
      TVT_SHAPER_ARRAY[15] (READ_ONLY),		
      TVT_SHAPER_ARRAY[16] (READ_ONLY),
      TVT_SHAPER_ARRAY[17] (READ_ONLY),
      TVT_SHAPER_ARRAY[18] (READ_ONLY),
      TVT_SHAPER_ARRAY[19] (READ_ONLY),
      TVT_SHAPER_ARRAY[20] (READ_ONLY),
      TVT_SHAPER_ARRAY[21] (READ_ONLY),
      TVT_SHAPER_ARRAY[22] (READ_ONLY),
      TVT_SHAPER_ARRAY[23] (READ_ONLY),
      TVT_SHAPER_ARRAY[24] (READ_ONLY),
      TVT_SHAPER_ARRAY[25] (READ_ONLY),
      TVT_SHAPER_ARRAY[26] (READ_ONLY),
      TVT_SHAPER_ARRAY[27] (READ_ONLY),
      TVT_SHAPER_ARRAY[28] (READ_ONLY),
      TVT_SHAPER_ARRAY[29] (READ_ONLY)
   }
}
/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a list of tvt points to be edited   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_page_tvt_shaper_list2
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "31 - 60";
   STYLE_PAGE
   ACCESS      OFFLINE;
   ITEMS
   {
      TVT_SHAPER_ARRAY[ 30] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 31] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 32] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 33] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 34] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 35] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 36] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 37] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 38] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 39] (READ_ONLY),
      TVT_SHAPER_ARRAY[40] (READ_ONLY),
      TVT_SHAPER_ARRAY[41] (READ_ONLY),
      TVT_SHAPER_ARRAY[42] (READ_ONLY),
      TVT_SHAPER_ARRAY[43] (READ_ONLY),      
      TVT_SHAPER_ARRAY[44] (READ_ONLY),
		COLUMNBREAK,
      TVT_SHAPER_ARRAY[45] (READ_ONLY),
      TVT_SHAPER_ARRAY[46] (READ_ONLY),
      TVT_SHAPER_ARRAY[47] (READ_ONLY),
      TVT_SHAPER_ARRAY[48] (READ_ONLY),
      TVT_SHAPER_ARRAY[49] (READ_ONLY),
      TVT_SHAPER_ARRAY[50] (READ_ONLY),
      TVT_SHAPER_ARRAY[51] (READ_ONLY),
      TVT_SHAPER_ARRAY[52] (READ_ONLY),
      TVT_SHAPER_ARRAY[53] (READ_ONLY),
      TVT_SHAPER_ARRAY[54] (READ_ONLY),
      TVT_SHAPER_ARRAY[55] (READ_ONLY),
      TVT_SHAPER_ARRAY[56] (READ_ONLY),
      TVT_SHAPER_ARRAY[57] (READ_ONLY),
      TVT_SHAPER_ARRAY[58] (READ_ONLY),
      TVT_SHAPER_ARRAY[59] (READ_ONLY)
   }
}
/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a list of tvt points to be edited   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_page_tvt_shaper_list3
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "61 - 90";
   STYLE_PAGE
   ACCESS      OFFLINE;
   ITEMS
   {
      TVT_SHAPER_ARRAY[60] (READ_ONLY),
      TVT_SHAPER_ARRAY[61] (READ_ONLY),
      TVT_SHAPER_ARRAY[62] (READ_ONLY),
      TVT_SHAPER_ARRAY[63] (READ_ONLY),
      TVT_SHAPER_ARRAY[64] (READ_ONLY),
      TVT_SHAPER_ARRAY[65] (READ_ONLY),
      TVT_SHAPER_ARRAY[66] (READ_ONLY),
      TVT_SHAPER_ARRAY[67] (READ_ONLY),
      TVT_SHAPER_ARRAY[68] (READ_ONLY),
      TVT_SHAPER_ARRAY[69] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 70] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 71] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 72] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 73] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 74] (READ_ONLY),
      COLUMNBREAK,
      TVT_SHAPER_ARRAY[ 75] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 76] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 77] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 78] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 79] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 80] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 81] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 82] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 83] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 84] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 85] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 86] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 87] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 88] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 89] (READ_ONLY)
   }
}
/*-----------------------------------------------------------------------------
   Description: Displays a tabbed page with a list of tvt points to be edited   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
MENU menu_page_tvt_shaper_list4
{
   LABEL       MENU_PAGE_TVT_SHAPER_LIST_LABEL + "91 - 120";
   STYLE_PAGE
   ACCESS      OFFLINE;
   ITEMS
   {
      TVT_SHAPER_ARRAY[ 90] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 91] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 92] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 93] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 94] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 95] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 96] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 97] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 98] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 99] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 100] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 101] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 102] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 103] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 104] (READ_ONLY),
		COLUMNBREAK,
      TVT_SHAPER_ARRAY[ 105] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 106] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 107] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 108] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 109] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 110] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 111] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 112] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 113] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 114] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 115] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 116] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 117] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 118] (READ_ONLY),
      TVT_SHAPER_ARRAY[ 119] (READ_ONLY),
   }
}

/* Visual component Definitions -----------------------------------------------*/

/*-----------------------------------------------------------------------------
   Description: Displays graph area, content is referenced from here   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
#ifdef EDDL_SUPPORT
GRAPH graph_tvt_shaper
{
    HEIGHT     X_LARGE;
    WIDTH      LARGE;
    X_AXIS     x_axis_tvt_shaper;
    MEMBERS
    {
        WAVEFORM_TVT_SHAPER_TVT,  waveform_tvt_shaper_tvt_profile;        
        WAVEFORM_TVT_SHAPER_ECHO, waveform_tvt_shaper_echo_profile;
    }
    VALIDITY ( var_local_tvt_shaper_profile_taken == TRUE );    
}

WAVEFORM waveform_tvt_shaper_echo_profile
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR TVT_SHAPER_ECHO_PROFILE_COLOR;     
   TYPE YT
   {
      X_INITIAL        var_local_tvt_shaper_x_axis_echo_start + 0.5;
      X_INCREMENT      var_local_tvt_shaper_x_axis_echo_increment;
      NUMBER_OF_POINTS TVT_SHAPER_ECHO_PROFILE_SIZE;
      Y_VALUES
      {
         TVT_SHAPER_ECHO_PROFILE_ARRAY
      }
   }
}

WAVEFORM waveform_tvt_shaper_tvt_profile
{
   Y_AXIS     y_axis_tvt_shaper;
   LINE_COLOR TVT_SHAPER_TVT_PROFILE_COLOR;              
   TYPE YT
   {
      X_INITIAL         -NUMBER_TVT_SHAPER_EXTRA_POINTS;
      X_INCREMENT       1;
      NUMBER_OF_POINTS  NUMBER_TVT_SHAPER_WORKING_POINTS;
      Y_VALUES
      {
         array_local_tvt_profile
      }
      KEY_POINTS
      {
         X_VALUES
         {
            var_local_tvt_shaper_index_display
         }
         Y_VALUES
         {
            array_local_tvt_profile[var_local_tvt_shaper_index_display + NUMBER_TVT_SHAPER_EXTRA_POINTS]
         }
      }
   }
}

AXIS x_axis_tvt_shaper
{
   LABEL         X_AXIS_TVT_SHAPER_LABEL;
   MIN_VALUE     var_local_tvt_shaper_x_axis_echo_start;
   MAX_VALUE     NUMBER_TVT_SHAPER_POINTS + 1;
   CONSTANT_UNIT X_AXIS_TVT_SHAPER_UNITS;
}

AXIS y_axis_tvt_shaper
{
    LABEL Y_AXIS_TVT_SHAPER_LABEL;
    MIN_VALUE TVT_SHAPER_Y_MIN;
    MAX_VALUE TVT_SHAPER_Y_MAX;
    CONSTANT_UNIT Y_AXIS_TVT_SHAPER_UNITS;
}
#endif


/* Method Definitions ---------------------------------------------------------*/
/*-----------------------------------------------------------------------------
   Description: This method is displayed under the "Device" menu    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_menu_tvt_shaper
{
   LABEL      METHOD_MENU_TVT_SHAPER_LABEL;
   ACCESS     OFFLINE;
   CLASS      INPUT;
   DEFINITION
   {
      long dialogResult;
      int  point;

      // Make a local copy so we can restore data      
      method_backup_tvt_shaper();
                                     
      // Add an abort method so we can restore the data
      add_abort_method(method_restore_tvt_shaper);
      
      // Grab an initial echo profile to use as a reference
      method_tvt_shaper_measure();

      // Turn on the mode because points may base validity on mode
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;
      
      // Display the dialog      
      MenuDisplay(menu_dialog_tvt_shaper, TVT_SHAPER_DIALOG_BUTTONS, dialogResult);

      // Dialog was closed, continue based on user selection 
      switch( dialogResult )
      {
         case 0:  // APPLY_BUTTON
            method_transfer_tvt_shaper_to_device();   
            break;            
         case 1:  // CANCEL_BUTTON
            method_restore_tvt_shaper();
      }
           
      // Remove the abort method that was added earlier
      remove_abort_method(method_restore_tvt_shaper);       
   }
}

/*-----------------------------------------------------------------------------
   Description: Make a backup copy of the data we are modifying so it can be 
                restored if needed    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_backup_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Make a local copy so we can restore data      
      var_backup_tvt_shaper_mode = TVT_SHAPER_MODE;

      for( point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++ )
      {
         array_backup_tvt_shaper_points[point] = TVT_SHAPER_ARRAY[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description: Restore the data we are modifying from the backup copy    
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_restore_tvt_shaper
{
   DEFINITION
   {
      int point;

      // Restore data from local copy      
      TVT_SHAPER_MODE = var_backup_tvt_shaper_mode;

      for( point = 0; point < NUMBER_TVT_SHAPER_POINTS; point++ )
      {
         TVT_SHAPER_ARRAY[point] = array_backup_tvt_shaper_points[point];
      }
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_set_tvt_shaper_index_display
{
   DEFINITION
   {
      var_local_tvt_shaper_index_display = var_local_tvt_shaper_index + 1;
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_index_display
{
   DEFINITION
   {
      if (var_local_tvt_shaper_index_display > var_local_tvt_shaper_index_display.MAX_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MAX_VALUE;
      }
      if (var_local_tvt_shaper_index_display < var_local_tvt_shaper_index_display.MIN_VALUE)
      {
         var_local_tvt_shaper_index_display = var_local_tvt_shaper_index_display.MIN_VALUE;
      }
      var_local_tvt_shaper_index = var_local_tvt_shaper_index_display - 1;
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_set_tvt_shaper_offset_display
{
   DEFINITION
   {
      var_local_tvt_shaper_value_display = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index];
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_get_tvt_shaper_offset_display
{
   DEFINITION
   {
      if (var_local_tvt_shaper_value_display > TVT_SHAPER_MAXIMUM_VALUE)
      {
         var_local_tvt_shaper_value_display = TVT_SHAPER_MAXIMUM_VALUE;
      }
      if (var_local_tvt_shaper_value_display < TVT_SHAPER_MINIMUM_VALUE)
      {
         var_local_tvt_shaper_value_display = TVT_SHAPER_MINIMUM_VALUE;
      }
      TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = var_local_tvt_shaper_value_display;
      method_adjust_tvt_value_at_current_index();
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_plus
{
   LABEL      METHOD_TVT_INDEX_PLUS_LABEL;
   DEFINITION
   {
      if(var_local_tvt_shaper_index < (NUMBER_TVT_SHAPER_POINTS - 1))
      {
         var_local_tvt_shaper_index = var_local_tvt_shaper_index + 1;
      }
      method_set_tvt_shaper_index_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_index_minus
{
   LABEL      METHOD_TVT_INDEX_MINUS_LABEL;
   DEFINITION
   {
      if(var_local_tvt_shaper_index > 0)
      {
          var_local_tvt_shaper_index = var_local_tvt_shaper_index - 1;
      }
      method_set_tvt_shaper_index_display();
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_increase
{
   LABEL      METHOD_TVT_VALUE_INCREASE_LABEL;
   DEFINITION
   {
      TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] + 1;
      if (TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] > TVT_SHAPER_MAXIMUM_VALUE)
      {
         TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_MAXIMUM_VALUE;
      }
      if (TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] < TVT_SHAPER_MINIMUM_VALUE)
      {
         TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_MINIMUM_VALUE;
      }
      var_local_tvt_shaper_value_display = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index];
      method_adjust_tvt_value_at_current_index();
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_value_decrease
{
   LABEL       METHOD_TVT_VALUE_DECREASE_LABEL;
   DEFINITION
   {
      TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] - 1;
      if (TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] > TVT_SHAPER_MAXIMUM_VALUE)
      {
         TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_MAXIMUM_VALUE;
      }
      if (TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] < TVT_SHAPER_MINIMUM_VALUE)
      {
         TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] = TVT_SHAPER_MINIMUM_VALUE;
      }
      var_local_tvt_shaper_value_display = TVT_SHAPER_ARRAY[var_local_tvt_shaper_index];
      method_adjust_tvt_value_at_current_index();
   }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_adjust_tvt_value_at_current_index
{
   DEFINITION
   {
      int current_index;

      current_index = (var_local_tvt_shaper_index_display * var_local_tvt_shaper_tvt_increment) + var_local_tvt_shaper_tvt_start;
      if (current_index < 0)
      {
         current_index = 0;
      }
      if (current_index >= (TVT_SHAPER_TVT_PROFILE_SIZE - 1))
      {
         current_index = TVT_SHAPER_TVT_PROFILE_SIZE - 2;
      }
      array_local_tvt_profile[var_local_tvt_shaper_index_display + NUMBER_TVT_SHAPER_EXTRA_POINTS] = TVT_SHAPER_TVT_PROFILE_ARRAY[current_index] + (TVT_SHAPER_ARRAY[var_local_tvt_shaper_index] * TVT_SHAPER_STEP_SIZE);
   }
}


/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_measure
{
  DEFINITION
  {
      int tvt_shaper_mode;
      float profile_increment;
      
      CMD_READ_TVT_SHAPER_MODE;
      tvt_shaper_mode = TVT_SHAPER_MODE;
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_OFF;
      CMD_WRITE_TVT_SHAPER_MODE;

      DELAY_TIME(2);
      
      METHOD_TVT_SHAPER_READ_PROFILE_DATA();

      TVT_SHAPER_MODE = tvt_shaper_mode;
      CMD_WRITE_TVT_SHAPER_MODE;

      // Turn on the mode because points may base validity on mode
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;

      // Compute echo profile values
      profile_increment = (TVT_SHAPER_X_MAX - TVT_SHAPER_X_MIN) / TVT_SHAPER_ECHO_PROFILE_SIZE;
      var_local_tvt_shaper_x_axis_echo_increment = NUMBER_TVT_SHAPER_POINTS / (TVT_SHAPER_X_MAX / profile_increment);
      var_local_tvt_shaper_x_axis_echo_start     = (TVT_SHAPER_X_MIN / profile_increment) * var_local_tvt_shaper_x_axis_echo_increment;
      
      // Compute tvt values
      var_local_tvt_shaper_tvt_start     = -(TVT_SHAPER_X_MIN / (TVT_SHAPER_X_MAX - TVT_SHAPER_X_MIN)) * TVT_SHAPER_TVT_PROFILE_SIZE;
      var_local_tvt_shaper_tvt_increment = (TVT_SHAPER_TVT_PROFILE_SIZE - var_local_tvt_shaper_tvt_start) / NUMBER_TVT_SHAPER_POINTS;
      
      method_tvt_shaper_update_local_tvt();
         
      var_local_tvt_shaper_profile_taken = TRUE;
  }
}

/*-----------------------------------------------------------------------------
   Description:     
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_update_local_tvt
{
  DEFINITION
  {
      int i;
      int current_index;

      for( i = -NUMBER_TVT_SHAPER_EXTRA_POINTS; i <= NUMBER_TVT_SHAPER_POINTS; i++ )
      {
         current_index = (i * var_local_tvt_shaper_tvt_increment) + var_local_tvt_shaper_tvt_start;
         if (current_index < 0)
         {
            current_index = 0;
         }
         if (current_index >= (TVT_SHAPER_TVT_PROFILE_SIZE - 1))
         {
            current_index = TVT_SHAPER_TVT_PROFILE_SIZE - 2;
         }

         if (i > 0)
         {
            array_local_tvt_profile[i + NUMBER_TVT_SHAPER_EXTRA_POINTS] = TVT_SHAPER_TVT_PROFILE_ARRAY[current_index] + (TVT_SHAPER_ARRAY[i-1] * TVT_SHAPER_STEP_SIZE);
         }
         else 
         {
            array_local_tvt_profile[i + NUMBER_TVT_SHAPER_EXTRA_POINTS] = TVT_SHAPER_TVT_PROFILE_ARRAY[current_index];
         }
      }
   }   
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper to the device 
                It writes both the mode and the points   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_transfer_tvt_shaper_to_device
{
   LABEL METHOD_TRANSFER_TVT_SHAPER_TO_DEVICE_LABEL; 
   DEFINITION
   {
      int point;
      
      TVT_SHAPER_MODE = TVT_SHAPER_MODE_ON;
      CMD_WRITE_TVT_SHAPER_MODE;
      CMD_WRITE_TVT_SHAPER_POINTS;
      PUT_MESSAGE(TVT_SHAPER_SUCCESSFUL_LABEL);
   }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper from the device 
                It reads both the mode and the points   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_transfer_tvt_shaper_from_device
{
   DEFINITION
   {
      CMD_READ_TVT_SHAPER_MODE;
      CMD_READ_TVT_SHAPER_POINTS;
      method_tvt_shaper_update_local_tvt();
      //PUT_MESSAGE("Read from device");
   }
}

/*-----------------------------------------------------------------------------
   Description: This is the method that transfers the shaper from the device 
                It reads both the mode and the points   
   Created by:
   Revised by:
 ------------------------------------------------------------------------------*/
METHOD method_tvt_shaper_update_from_device
{
   LABEL METHOD_TVT_SHAPER_UPDATE_FROM_DEVICE_LABEL;
   DEFINITION
   {
      method_transfer_tvt_shaper_from_device();
      method_tvt_shaper_measure();
   }
}

/*-----------------------------------------------------------------------------
                                                         For Internal Use Only.
      Copyright (C) Siemens-Milltronics Process Instruments Inc.
      All Rights Reserved.

End of file -------------------------------------------------------------------*/
